/*
 * File: libppr.h
 * Description: An interface for libppr library
 * Author: Stanislaw Galus
 * Created: 28.X.2003
 * Revision: $Id: libppr.h,v 0.11 2003/12/28 18:30:49 sgalus Exp $
 */

/*
 * Include stdio.h before including this file.
 */

#ifndef libppr_h
#define libppr_h

#define N_CHARSETS 3
#define MAX_CHAR 0xff
#define UNASSG 0xffff

#define MAXSTR 500
#define UESCL 6                 /* length of Unicode escape */
#define UESCD 4                 /* number of digits in Unicode escape */
#define STDIOS "-"              /* name for standard input and output */
#define NORMAL 0                /* macros for mode */
#define EXTENDED 1
#define BUILDWF 2
#define ISO88592 0              /* macros for encoding */
#define CP852 1
#define WINDOWS1250 2
#define PPR_OK 0                /* codes returned by pprrun */
#define PPR_ERR_OPENING_OUTPUT_FILE 1
#define PPR_ERR_WRITING_OUTPUT_FILE 2
#define PPR_ERR_CLOSING_FILE 3
#define PPR_ERR_CHECKING_WORD 4
#define PPR_ERR_OPENING_DIAGFILE 5
#define PPR_ERR_TOO_MANY_INPUT_FILES 6
#define PPR_ERR_PSWF 7

#define NINFL 0                 /* substantival declension */
#define AINFL 1                 /* adjectival declension */
#define VINFL 2                 /* conjugation */
#define INFLS 3                 /* number of inflexions */
#define VOID 0                  /* parts of speech */
#define NOUN 1
#define ADJECTIVE 2
#define PRONOUN 3
#define NUMERAL 4
#define VERB 5
#define PSSP 6
#define ACTP 7
#define VN 8
#define PSTP 9
#define ADVERB 10
#define PREPOSITION 11
#define CONJUNCTION 12
#define PARTICLE 13
#define INTERJECTION 14
#define MASCULINE_PERSONAL 1    /* genders */
#define MASCULINE_ANIMAL 2
#define MASCULINE_INANIMATE 3
#define FEMININE 4
#define NEUTER 5
#define POSITIVE 1              /* degrees */
#define COMPARATIVE 2
#define IMPERFECT 1             /* aspects */
#define PERFECT 2
#define NOFORM "-"
#define UNINFLECTED -1          /* lexeme.ending for uninflected lexemes
                                   and return value from infls */
#define NELEMS(a) (sizeof(a) / sizeof(a[0]))
#define HALT()                                          \
  do                                                    \
  {                                                     \
    fprintf(stderr,                                     \
            "Error at %s, line %d. Program aborted.\n", \
            __FILE__,                                   \
            __LINE__);                                  \
    exit(EXIT_FAILURE);                                 \
  } while (0)
#define POS(t) ((t) & 15)       /* part of speech for lexeme.type */
#define SEL(t) (((t) >> 4) & 7) /* selective category for lexeme.type */
#define TYPE(s, p) (16 * s + p) /* lexeme.type */

typedef struct namelist         /* struct for storing names of files */
{
  char *name;
  struct namelist *next;
} namelist;

typedef struct lexeme
{
  char type;                    /* type of a lexeme; see below */
  char *stem;                   /* its stem */
  short ending;                 /* index of its ending */
} lexeme;

/* The four less significant bits of lexeme.type denote its part of
   speech (0 - 14, as specified by the table posp). The next three bits
   denote a value of a selective category of the lexeme: gender for
   nouns (0 - 5, as specified by the table gender), degree for
   adjectives and adverbs (0 - 2, as specified by the table degree), and
   aspect for verbs (0 - 2, as specified by the table aspect). The value
   of part of speech of 15 is erroneous, as well as values of categories
   that are out of range. The most significant bit of lexeme.type is not
   used. Example: if it is a feminine noun, there should be POS(type) ==
   NOUN and SEL(type) == FEMININE.

   lexeme.type:
         +------+------+------+------+------+------+------+------+
   i:    |   7  |   6  |   5  |   4  |   3  |   2  |   1  |   0  |
   2^i:  | 128  |  64  |  32  |  16  |   8  |   4  |   2  |   1  |
   b[i]: | b[7] | b[6] | b[5] | b[4] | b[3] | b[2] | b[1] | b[0] |
         +------+------+------+------+------+------+------+------+
         |      | selective category |      part of speech       |
         +------+--------------------+---------------------------+
   part of speech = b[3]*2^3 + b[2]*2^2 + b[1]*2^1 + b[0] * 2^0
                  = POS(lexeme.type)
   selective category = b[6]*2^2 + b[5]*2^1 + b[4]*2^0
                      = SEL(lexeme.type) */

typedef struct dictionary
{
  lexeme *lextab;               /* table of lexemes */
  int nlex;                     /* current number of lexemes */
  int maxlex;                   /* maximum number of lexemes */
  char ***endtab[INFLS];        /* table of endings */
  int nend[INFLS];              /* current numbers of endings */
  int maxend[INFLS];            /* maximum numbers of endings */
  char *stemaddr;               /* location for stems */
  char *endaddr[INFLS];         /* location for endings */
} dictionary;

typedef struct dictlist
{
  dictionary *d;
  struct dictlist *next;
} dictlist;

extern const unsigned short int chcode[N_CHARSETS][MAX_CHAR + 1];

extern char *program_name;
extern int encoding;
extern int mode;
extern char *outfn;
extern namelist *wfns;
extern namelist *ifns;
extern char newline;
extern char carret;
extern char space;
#ifdef DIAGNOSTICS
extern FILE *dgf;
#endif

int chalnum(int ch, int c);
int chalpha(int ch, int c);
int chcntrl(int ch, int c);
int chdigit(int ch, int c);
int chgraph(int ch, int c);
int chlower(int ch, int c);
int chprint(int ch, int c);
int chpunct(int ch, int c);
int chspace(int ch, int c);
int chupper(int ch, int c);
int chxdigit(int ch, int c);
int chtolower(int ch, int c);
int chtoupper(int ch, int c);
int chxtoi(int ch, int c);
int chitox(int ch, int c);

int chkwrd(const char *s);

void freedl(void);
/* int wswf(const dictionary *d, const char *fn); */
int pswf(FILE *f0, FILE *f1);
int riwf(const char *fn);
int dsearch(const char *s);
int ffolod(const char *s, dictlist **d, int *p);
int fnolod(const char *s, dictlist **d, int *p);

void wprintf(const char *format, ...);
int pprbegin(void);
void pprend(void);
int pprrun(void);
char *toiso(int ch, const char *s, char *t);
void tounique(const char *s, char *t);

#endif /* libppr_h */
